@@ -2,11 +2,9 @@
 This module includes functions to process request headers.
 Refer to the PLAYWRIGHT_PROCESS_REQUEST_HEADERS setting for more information.
 """
-import warnings
 from urllib.parse import urlparse
 
 from playwright.async_api import Request as PlaywrightRequest
-from scrapy.exceptions import ScrapyDeprecationWarning
 from scrapy.http.headers import Headers
 
 
@@ -34,18 +32,3 @@ async def use_scrapy_headers(
     if scrapy_headers_str.get("user-agent"):
         playwright_headers["user-agent"] = scrapy_headers_str["user-agent"]
     return playwright_headers
-
-
-async def use_playwright_headers(
-    browser_type: str,
-    playwright_request: PlaywrightRequest,
-    scrapy_headers: Headers,
-) -> dict:
-    warnings.warn(
-        "The 'scrapy_playwright.headers.use_playwright_headers' function is"
-        " deprecated, please set 'PLAYWRIGHT_PROCESS_REQUEST_HEADERS=None'"
-        " instead.",
-        category=ScrapyDeprecationWarning,
-        stacklevel=1,
-    )
-    return await playwright_request.all_headers()
